@@ -84,7 +84,7 @@ mark_qobject_children(QObject * qobject)
 		++it;
 		obj = getPointerObject(child);
 		if (obj != Qnil) {
-			if(do_debug & qtdb_gc) printf("Marking (%s*)%p -> %p\n", child->className(), child, (void*)obj);
+			if(do_debug & qtdb_gc) qWarning("Marking (%s*)%p -> %p\n", child->className(), child, (void*)obj);
 			rb_gc_mark(obj);
 		}
 		
@@ -99,7 +99,7 @@ smokeruby_mark(void * p)
     smokeruby_object * o = (smokeruby_object *) p;
     const char *className = o->smoke->classes[o->classId].className;
 	
-	if(do_debug & qtdb_gc) printf("Checking for mark (%s*)%p\n", className, o->ptr);
+	if(do_debug & qtdb_gc) qWarning("Checking for mark (%s*)%p\n", className, o->ptr);
 		
     if(o->ptr && o->allocated) {
 		if (isDerivedFromByName(o->smoke, className, "QListView")) {
@@ -111,7 +111,7 @@ smokeruby_mark(void * p)
 				++it;
 				obj = getPointerObject(item);
 				if (obj != Qnil) {
-					if(do_debug & qtdb_gc) printf("Marking (%s*)%p -> %p\n", className, item, (void*)obj);
+					if(do_debug & qtdb_gc) qWarning("Marking (%s*)%p -> %p\n", className, item, (void*)obj);
 					rb_gc_mark(obj);
 				}
 			}
@@ -127,7 +127,7 @@ smokeruby_mark(void * p)
 					item = table->item(row, col);
 					obj = getPointerObject(item);
 					if (obj != Qnil) {
-						if(do_debug & qtdb_gc) printf("Marking (%s*)%p -> %p\n", className, item, (void*)obj);
+						if(do_debug & qtdb_gc) qWarning("Marking (%s*)%p -> %p\n", className, item, (void*)obj);
 						rb_gc_mark(obj);
 					}
 				}
@@ -141,12 +141,23 @@ smokeruby_mark(void * p)
     		for ( QCanvasItemList::iterator it = list.begin(); it != list.end(); ++it ) {
 				obj = getPointerObject(*it);
 				if (obj != Qnil) {
-					if(do_debug & qtdb_gc) printf("Marking (%s*)%p -> %p\n", className, *it, (void*)obj);
+					if(do_debug & qtdb_gc) qWarning("Marking (%s*)%p -> %p\n", className, *it, (void*)obj);
 					rb_gc_mark(obj);
 				}
 			}
 			return;
 		}
+
+		if (isDerivedFromByName(o->smoke, className, "QCanvasItem")) {
+			QCanvasItem * item = (QCanvasItem *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QCanvasItem"));
+			QCanvas * canvas = item->canvas();
+			obj = getPointerObject(canvas);
+			if (obj != Qnil) {
+				if(do_debug & qtdb_gc) qWarning("Marking (%s*)%p -> %p\n", "QCanvas", canvas, (void*)obj);
+				rb_gc_mark(obj);
+			}
+			return;
+		}
 		
 		if (isDerivedFromByName(o->smoke, className, "QObject")) {
 			QObject * qobject = (QObject *) o->smoke->cast(o->ptr, o->classId, o->smoke->idClass("QObject"));
@@ -162,7 +173,7 @@ smokeruby_free(void * p)
     smokeruby_object *o = (smokeruby_object*)p;
     const char *className = o->smoke->classes[o->classId].className;
 	
-	if(do_debug & qtdb_gc) printf("Checking for delete (%s*)%p allocated: %s\n", className, o->ptr, o->allocated ? "true" : "false");
+	if(do_debug & qtdb_gc) qWarning("Checking for delete (%s*)%p allocated: %s\n", className, o->ptr, o->allocated ? "true" : "false");
     
 	if(application_terminated || !o->allocated || o->ptr == 0) {
 		free(o);
@@ -175,6 +186,11 @@ smokeruby_free(void * p)
 	if (	strcmp(className, "QObject") == 0
 			|| strcmp(className, "QListBoxItem") == 0
 			|| strcmp(className, "QStyleSheetItem") == 0
+			|| strcmp(className, "KCommand") == 0
+			|| strcmp(className, "KNamedCommand") == 0
+			|| strcmp(className, "KMacroCommand") == 0
+			|| strcmp(className, "KAboutData") == 0
+			|| strcmp(className, "KCmdLineArgs") == 0
 			|| strcmp(className, "QSqlCursor") == 0 )
 	{
 		// Don't delete instances of these classes for now
@@ -230,7 +246,7 @@ smokeruby_free(void * p)
 		}
 	}
 			
-	if(do_debug & qtdb_gc) printf("Deleting (%s*)%p\n", className, o->ptr);
+	if(do_debug & qtdb_gc) qWarning("Deleting (%s*)%p\n", className, o->ptr);
 	
 	char *methodName = new char[strlen(className) + 2];
 	methodName[0] = '~';
@@ -406,6 +422,7 @@ construct_copy(smokeruby_object *o)
     Smoke::Index ccMeth = o->smoke->findMethod(o->classId, ccId);
 
     if(!ccMeth) {
+	delete[] ccArg;
 	return 0;
     }
 	Smoke::Index method = o->smoke->methodMaps[ccMeth].method;
@@ -701,7 +718,7 @@ marshall_basetype(Marshall *m)
 		
 		obj = set_obj_info(classname, o);
 		if (do_debug & qtdb_calls) {
-			printf("allocating %s %p -> %p\n", classname, o->ptr, (void*)obj);
+			qWarning("allocating %s %p -> %p\n", classname, o->ptr, (void*)obj);
 		}
 
 		if(m->type().isStack()) {
@@ -731,17 +748,21 @@ static void marshall_unknown(Marshall *m) {
 
 static void marshall_charP(Marshall *m) {
     switch(m->action()) {
-      case Marshall::FromVALUE:
-	{
-	    VALUE rv = *(m->var());
-	    if(rv == Qnil) {
-                m->item().s_voidp = 0;
-                break;
-	    }
-		
-        m->item().s_voidp = StringValuePtr(rv);
-	}
-	break;
+    case Marshall::FromVALUE:
+    {
+        VALUE rv = *(m->var());
+	    if (rv == Qnil) {
+            m->item().s_voidp = 0;
+            break;
+        }
+	
+        int len = RSTRING(rv)->len;
+        char* mem = (char*) malloc(len+1);
+        memcpy(mem, StringValuePtr(rv), len);
+        mem[len] ='\0';
+        m->item().s_voidp = mem;
+    }
+        break;
       case Marshall::ToVALUE:
 	{
 	    char *p = (char*)m->item().s_voidp;
@@ -761,14 +782,18 @@ static void marshall_charP(Marshall *m) {
 
 void marshall_ucharP(Marshall *m) {
     switch(m->action()) {
-      case Marshall::FromVALUE:
-	{
-	    VALUE rv = *(m->var());
-	    if(rv == Qnil) {
-		m->item().s_voidp = 0;
-		break;
-	    }
-        m->item().s_voidp = StringValuePtr(rv);
+    case Marshall::FromVALUE:
+    {
+        VALUE rv = *(m->var());
+        if (rv == Qnil) {
+            m->item().s_voidp = 0;
+            break;
+        }
+        int len = RSTRING(rv)->len;
+        char* mem = (char*) malloc(len+1);
+        memcpy(mem, StringValuePtr(rv), len);
+        mem[len] ='\0';
+        m->item().s_voidp = mem;
 	}
 	break;
       case Marshall::ToVALUE:
@@ -831,18 +856,18 @@ rstringFromQString(QString * s) {
 }
 
 static void marshall_QString(Marshall *m) {
-    switch(m->action()) {
-      case Marshall::FromVALUE:
+	switch(m->action()) {
+	case Marshall::FromVALUE:
 	{
-	    QString* s = 0;
-	    if( *(m->var()) != Qnil) {
-               s = qstringFromRString(*(m->var()));
-            } else {
-                s = new QString(QString::null);
-            }
+		QString* s = 0;
+		if ( *(m->var()) != Qnil) {
+			s = qstringFromRString(*(m->var()));
+		} else {
+			s = new QString(QString::null);
+		}
 		
-	    m->item().s_voidp = s;
-	    m->next();
+		m->item().s_voidp = s;
+		m->next();
 		
 		if (!m->type().isConst() && *(m->var()) != Qnil && s != 0 && !s->isNull()) {
 			rb_str_resize(*(m->var()), 0);
@@ -850,87 +875,50 @@ static void marshall_QString(Marshall *m) {
 			rb_str_cat2(*(m->var()), StringValuePtr(temp));
 		}
 	    
-		if(s && m->type().isConst() && m->cleanup())
-		delete s;
+		if (s != 0 && m->cleanup()) {
+			delete s;
+		}
 	}
 	break;
-      case Marshall::ToVALUE:
+	case Marshall::ToVALUE:
 	{
-	    QString *s = (QString*)m->item().s_voidp;
-	    if(s) {
-	    	if (s->isNull()) {
-                    *(m->var()) = Qnil;
-	     	} else {
-               *(m->var()) = rstringFromQString(s);
-	     	}
-	     	if(m->cleanup() || m->type().isStack())
-	     	delete s;
-         } else {
-                *(m->var()) = Qnil;
-            }
+		QString *s = (QString*)m->item().s_voidp;
+		if (s != 0) {
+			if (s->isNull()) {
+				*(m->var()) = Qnil;
+			} else {
+				*(m->var()) = rstringFromQString(s);
+			}
+			if (m->cleanup() || m->type().isStack()) {
+				delete s;
+			}
+		} else {
+			*(m->var()) = Qnil;
+		}
 	}
 	break;
-      default:
-	m->unsupported();
-	break;
+	default:
+		m->unsupported();
+		break;
     }
 }
 
-static void marshall_QByteArray(Marshall *m) {
-    switch(m->action()) {
-      case Marshall::FromVALUE:
-	{
-	    VALUE rv = *(m->var());
-	    QByteArray *s = 0;
-		VALUE data = Qnil;
-	    if(rv != Qnil) {
-			if (rb_respond_to(rv, rb_intern("data")) != 0) {
-				// Qt::ByteArray - use the contents of the 'data' instance var
-				data = rb_funcall(qt_internal_module, rb_intern("get_qbytearray"), 1, rv);
-				if (TYPE(data) == T_DATA) {
-					// A C++ QByteArray inside the Qt::ByteArray
-					Data_Get_Struct(data, QByteArray, s);
-				} else {
-					// Or a ruby String inside
-            		s = new QByteArray(RSTRING(data)->len);
-					memcpy((void*)s->data(), StringValuePtr(data), RSTRING(data)->len);
-				}
-			} else {
-				// Ordinary ruby String - use the contents of the string
-            	s = new QByteArray(RSTRING(rv)->len);
-				memcpy((void*)s->data(), StringValuePtr(rv), RSTRING(rv)->len);
-			}
-        } else {
-            s = new QByteArray(0);
-	    }
-	    m->item().s_voidp = s;
-	    
-		m->next();
-	    
-		if(s && m->cleanup() && data == Qnil)
-		delete s;
+// The only way to convert a QChar to a QString is to
+// pass a QChar to a QString constructor. However,
+// QStrings aren't in the QtRuby api, so add this
+// convenience method 'Qt::Char.to_s' to get a ruby
+// string from a Qt::Char.
+VALUE
+qchar_to_s(VALUE self)
+{
+	smokeruby_object *o = value_obj_info(self);
+	if (o == 0 || o->ptr == 0) {
+		return Qnil;
 	}
-	break;
-      case Marshall::ToVALUE:
-	{
-		VALUE result;
-	    QByteArray *s = (QByteArray*)m->item().s_voidp;
-	    if(s) {
-			VALUE string = rb_str_new2("");
-			rb_str_cat(string, (const char *)s->data(), s->size());
-			result = rb_funcall(qt_internal_module, rb_intern("create_qbytearray"), 2, string, Data_Wrap_Struct(rb_cObject, 0, 0, s));
-        } else {
-			result = Qnil;
-		}
-		*(m->var()) = result;
-	    if(m->cleanup())
-		delete s;
-	}
-	break;
-      default:
-	m->unsupported();
-	break;
-    }
+
+	QChar * qchar = (QChar*) o->ptr;
+	QString s(*qchar);
+	return rstringFromQString(&s);
 }
 
 #if 0
@@ -1242,8 +1230,8 @@ void marshall_QStringList(Marshall *m) {
 		    	rb_ary_push(list, rstringFromQString(&(*it)));
 		}
 			
-		if (stringlist != 0 && m->type().isConst() && m->cleanup())
-		delete stringlist;
+		if (m->cleanup())
+			delete stringlist;
 	    break;
       }
       case Marshall::ToVALUE: 
@@ -1297,11 +1285,14 @@ void marshall_QStrList(Marshall *m) {
 	    m->item().s_voidp = stringlist;
 	    m->next();
 
-	    if(m->cleanup()) {
-		rb_ary_clear(list);
-		for(const char * it = stringlist->first(); it != 0; it = stringlist->next())
-		    rb_ary_push(list, rb_str_new2(it));
-		delete stringlist;
+		if (!m->type().isConst()) {
+			rb_ary_clear(list);
+			for(const char * it = stringlist->first(); it != 0; it = stringlist->next())
+				rb_ary_push(list, rb_str_new2(it));
+		}
+
+		if (m->cleanup()) {
+			delete stringlist;
 	    }
 	    break;
       }
@@ -1362,15 +1353,19 @@ void marshall_ItemList(Marshall *m) {
 	    m->item().s_voidp = cpplist;
 	    m->next();
 
-	    if(m->cleanup()) {
-		rb_ary_clear(list);
-		for(ItemListIterator it = cpplist->begin();
-		    it != cpplist->end();
-		    ++it) {
-		    VALUE obj = getPointerObject((void*)(*it));
-		    rb_ary_push(list, obj);
+		if (!m->type().isConst()) {
+			rb_ary_clear(list);
+			for(ItemListIterator it = cpplist->begin();
+				it != cpplist->end();
+				++it ) 
+			{
+				VALUE obj = getPointerObject((void*)(*it));
+				rb_ary_push(list, obj);
+			}
 		}
-		delete cpplist;
+
+		if (m->cleanup()) {
+			delete cpplist;
 	    }
 	}
 	break;
@@ -1442,13 +1437,16 @@ void marshall_QValueListInt(Marshall *m) {
 	    m->item().s_voidp = valuelist;
 	    m->next();
 
-	    if(m->cleanup()) {
-		rb_ary_clear(list);
-		for(QValueListIterator<int> it = valuelist->begin();
-		    it != valuelist->end();
-		    ++it)
-		    rb_ary_push(list, INT2NUM((int)*it));
-		delete valuelist;
+		if (!m->type().isConst()) {
+			rb_ary_clear(list);
+			for(QValueListIterator<int> it = valuelist->begin();
+				it != valuelist->end();
+				++it)
+				rb_ary_push(list, INT2NUM((int)*it));
+		}
+
+		if (m->cleanup()) {
+			delete valuelist;
 	    }
 	}
 	break;
@@ -1786,15 +1784,19 @@ void marshall_ValueItemList(Marshall *m) {
 	    m->item().s_voidp = cpplist;
 	    m->next();
 
-	    if(m->cleanup()) {
-		rb_ary_clear(list);
-		for(ItemListIterator it = cpplist->begin();
-		    it != cpplist->end();
-		    ++it) {
-		    VALUE obj = getPointerObject((void*)&(*it));
-		    rb_ary_push(list, obj);
+		if (!m->type().isConst()) {
+			rb_ary_clear(list);
+			for(ItemListIterator it = cpplist->begin();
+				it != cpplist->end();
+				++it) 
+			{
+				VALUE obj = getPointerObject((void*)&(*it));
+				rb_ary_push(list, obj);
+			}
 		}
-		delete cpplist;
+
+		if (m->cleanup()) {
+			delete cpplist;
 	    }
 	}
 	break;
@@ -1845,8 +1847,8 @@ void marshall_ValueItemList(Marshall *m) {
     }
 }
 
-#define DEF_VALUELIST_MARSHALLER(ListIdent,ItemList,Item,Itr) namespace { char ListIdent##STR[] = #Item; };  \
-        Marshall::HandlerFn marshall_##ListIdent = marshall_ValueItemList<Item,ItemList,Itr,ListIdent##STR>;
+#define DEF_VALUELIST_MARSHALLER(ListIdent,ItemList,Item,Itr) namespace dummy { char ListIdent##STR[] = #Item; };  \
+        Marshall::HandlerFn marshall_##ListIdent = marshall_ValueItemList<Item,ItemList,Itr,dummy::ListIdent##STR>;
 
 DEF_VALUELIST_MARSHALLER( QVariantList, QValueList<QVariant>, QVariant, QValueList<QVariant>::Iterator )
 DEF_VALUELIST_MARSHALLER( QPixmapList, QValueList<QPixmap>, QPixmap, QValueList<QPixmap>::Iterator )
@@ -1883,6 +1885,7 @@ TypeHandler Qt_handlers[] = {
     { "Q_UINT64&", marshall_ulonglong },
     { "Q_ULLONG", marshall_ulonglong },
     { "Q_ULLONG&", marshall_ulonglong },
+    { "signed int&", marshall_intR },
     { "int&", marshall_intR },
     { "int*", marshall_intR },
     { "bool&", marshall_boolR },
@@ -1895,8 +1898,6 @@ TypeHandler Qt_handlers[] = {
     { "QUObject*", marshall_QUObject },
     { "const QCOORD*", marshall_QCOORD_array },
     { "void", marshall_void },
-    { "QByteArray", marshall_QByteArray },
-    { "QByteArray&", marshall_QByteArray },
     { "QValueList<int>", marshall_QValueListInt },
     { "QValueList<int>&", marshall_QValueListInt },
     { "QValueList<QVariant>", marshall_QVariantList },
